
const fs = require('fs')
const path = __dirname + '/package.json'
const axios = require('axios')
const WebSocket = require('ws');
const uuidv4 = require('uuid').v4

let data = fs.readFileSync(path, { encoding: 'utf8', flag: 'r' })
let pkginfo = JSON.parse(data)

function sleep(ms) {
  return new Promise((resolve) => {
    setTimeout(resolve, ms);
  });
}

module.exports = {
    ...pkginfo,
    handler: function(db, socket, redisClient, request, message, exported) {
        // socket handler
      console.log('Inside hawk chatgpt handler')
      console.log(socket)

      let lmessage
      let r
      let data

      if (!('route' in message)) {
        lmessage = message
        lmessage.status = false
        lmessage.code = 404
        lmessage.details = 'No cmd field has been specified, unable to continue.'
        socket.send(JSON.stringify(lmessage))
        return
      }

      if (request.session.userProfile.isAdmin == false && request.session.userProfile.isSOC == false) {
        lmessage = message
        lmessage.status = false
        lmessage.code = 404
        lmessage.details = 'User has invalid permissions for access; must have SOC or Admin privileges.'
        socket.send(JSON.stringify(lmessage))
        return
      }

      const cmd = camelize(message.route)
      console.log(cmd)

      switch (cmd) {

        case 'hosts':
          if (!('data' in message)) {
            lmessage = message
            lmessage.status = false
            lmessage.code = 404
            lmessage.details = `No data found in message: ${message.route}`
            socket.send(JSON.stringify(lmessage))
            return
          }
          r = message
          module.exports.hosts(db, request.session, message.data, exported).then((x) => {
            r.data = x
            r.status = true
            r.code = 200
            console.log("Sending back results")
            console.log(x)
            r.details = `Successfully fetched results.`
            socket.send(JSON.stringify(r))
          })
            .catch((e) => {
              lmessage = message
              lmessage.status = false
              lmessage.code = 500
              lmessage.details = e.toString()
              socket.send(JSON.stringify(lmessage))
            })
          break

        case 'search':
          if (!('data' in message)) {
            lmessage = message
            lmessage.status = false
            lmessage.code = 404
            lmessage.details = `No data found in message: ${message.route}`
            socket.send(JSON.stringify(lmessage))
            return
          }
          r = message
          module.exports.search(db, request.session, message.data, exported).then((x) => {
            r.data = x
            r.status = true
            r.code = 200
            console.log("Sending back results")
            console.log(x)
            r.details = `Successfully fetched results.`
            socket.send(JSON.stringify(r))
          })
            .catch((e) => {
              lmessage = message
              lmessage.status = false
              lmessage.code = 500
              lmessage.details = e.toString()
              socket.send(JSON.stringify(lmessage))
            })
          break
        default:
          lmessage = message
          lmessage.status = false
          lmessage.code = 404
          lmessage.details = `No known route found matching ${message.route}`
          socket.send(JSON.stringify(lmessage))
          break
      }

    },
    registerRoutes: function(app) {
    },

};

const camelize = function (str) {
  return str.replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, (match, index) => {
    if (+match === 0) return '' // or if (/\s+/.test(match)) for white spaces
    return index === 0 ? match.toLowerCase() : match.toUpperCase()
  })
}

module.exports.hosts = function(db, session, record, exported) {
  // fetch the given credentials from vault, send err msg on failure
  // on success make call for categories and severities.

  return exported['credentials.getCredential'](record.group_id, record.credential_id, record.token, db, session).then((x) => {
    let xd = x.data
    if ('data' in xd && 'data' in xd.data && xd.data.data.tokenSecret) {
      const username = xd.data.data.username
      const password = xd.data.data.password
      // fetch action info by id
      return exported['actions.getEntry'](db, session, record.action_id).then((x) => {
         if (x.length == 0) throw new Error("No action vendor record found.")
         const escData = x[0]
         console.log(escData)
         // auth && get list of agents
         let url = escData.config.url
         if(url.toLowerCase().indexOf('portal.hawk.io') == -1) {
           url = escData.config.url
         } else url = 'https://portal.hawk.io:8080'

         // Make a POST request to the login endpoint
         return axios.post(url + '/API/1.1/login' , {
           username: username,
           password: password, 
           secure: "none"
         }, {
           // This will ensure that cookies are saved in the response
           withCredentials: true
         }).then((response) => {
           return axios.get(url + '/API/1.1/agents', { withCredentials: true }).then((response2) => {
             return response2.data
           }).catch((error) => {
             console.log(error.response.data)
           })
         }).catch((error) => {
           console.log(error.response.data)
         })

      })
    } else
      throw new Error("Credential token failed to match, unable to fetch credential secret.")
  })
}


module.exports.search = function(db, session, record, exported) {
  // fetch the given credentials from vault, send err msg on failure
  // on success make call for categories and severities.

  return exported['credentials.getCredential'](record.group_id, record.credential_id, record.token, db, session).then((x) => {
    let xd = x.data
    if ('data' in xd && 'data' in xd.data && xd.data.data.tokenSecret) {
      const username = xd.data.data.username
      const password = xd.data.data.password
      // fetch action info by id
      return exported['actions.getEntry'](db, session, record.action_id).then((x) => {
         if (x.length == 0) throw new Error("No action vendor record found.")
         const escData = x[0]
         console.log(escData)
         let url = escData.config.url
         if(url.toLowerCase().indexOf('portal.hawk.io') == -1) {
           url = escData.config.url
         } else url = 'https://portal.hawk.io:8080'

         // Make a POST request to the login endpoint
         return axios.post(url + '/API/1.1/login' , {
           username: username,
           password: password, 
           secure: "none"
         }, {
           // This will ensure that cookies are saved in the response
           withCredentials: true
         }).then((response) => {
           console.log(response.data)
           const cookies = response.headers['set-cookie'];
           console.log('Got cookies: ' + cookies)
           return new Promise((resolve, reject) => {
               const cookieHeader = cookies.join('; ');
               const ws = new WebSocket(url.replace('https','wss') + '/wsm', {
                 headers: {
                   Cookie: cookieHeader,
                 },
               });

               ws.on('open', function open() {
                 console.log('WebSocket connection established');
                 // You can now send messages with ws.send() and set up other event listeners
                 // get processes
                 data = { };
                 data['id'] = uuidv4()
                 data['cmd'] = 'queryAgent'
                 data['data'] = { type: 'processes', id: record.agent_uuid }
                 console.log('Querying for processes')
                 ws.send(JSON.stringify(data))
               });

               ws.on('message', function incoming(data) {
                 console.log('Received:', data);
                 // resolve !
                 ws.disconnect()
                 resolve(data)
               });

               ws.on('error', function error(err) {
                 console.log('WebSocket error:', err);
                 reject(err)
               });
           })

         })
      })
    } else
      throw new Error("Credential token failed to match, unable to fetch credential secret.")
  })
}

